(function (root, factory) {
  if (typeof define === 'function' && define.amd) {
    define([], factory);
  } else if (typeof exports === 'object' && typeof module !== "undefined") {
    module.exports = factory();
  } else {
    root.checkboxTree = factory();
  }
}(this, function () {
    var defaultLevel = {
      children: 'children',
      checkedProperty: 'flag',
      name: 'name',
      id: 'id',
      parentId: 'parent_id'
    };
    var checkboxTree = function (node, settings) {
      var level = this.extend(defaultLevel, settings.level);
      this.node = node;
      this.data = settings.data;
      this.childrenSign = level.children;
      this.checkedProperty = level.checkedProperty;
      this.nameSign = level.name;
      this.idSign = level.id;
      this.parentIdSign = level.parentId;
      this.changeHandler =  settings.changeHandler;
      this.dataMap = {};
      this.selectedData = [];
      this.getSelectedNodes = this._getSelectedNodes;
      this.init();
    }
    checkboxTree.prototype = {
      extend: function (def, add) {
        Object.keys(add).forEach(function (value) {
          def[value] = add[value];
        })
        return def;
      },
      init: function () {
        var checkboxTree = this.$createElement('div', { className: 'checkbox-tree' });
        checkboxTree.appendChild(this.getTreeDom(this.data));
        this.node.appendChild(checkboxTree);
        checkboxTree.addEventListener('click', this.clickHandler.bind(this));
      },
      getTreeDom: function (nodeData) {
        var fragment = document.createDocumentFragment();
        nodeData.forEach(function (value) {
          this.dataMap[value['id']] = value;
          var treeNode = this.$createElement('div', { className: 'tree-node' });
          if (value[this.childrenSign] && value[this.childrenSign].length) {
            var indeterminate = this.getIndeterminate(value[this.childrenSign], !value[this.checkedProperty]);
            treeNode.appendChild(this.getTreeNodeContent(value[this.nameSign], value[this.idSign], value[this.checkedProperty], true, indeterminate));
            treeNode.appendChild(this.getTreeNodeChild(this.getTreeDom(value[this.childrenSign])));
          } else {
            treeNode.appendChild(this.getTreeNodeContent(value[this.nameSign], value[this.idSign], value[this.checkedProperty], false, false));
          }
          fragment.appendChild(treeNode);
        }, this)
        return fragment;
      },
      getTreeNodeContent: function (text, id, checked, fill, indeterminate) {
        var emptySign = fill ? '' : 'empty';
        var node = this.$createElement('div', { className: 'tree-node-content' });
        node.appendChild(this.$createElement('span', { className: 'tree-icon ' + emptySign }));
        node.appendChild(this.$createElement('input', { type: 'checkbox', name: id, checked: checked, indeterminate: indeterminate }));
        node.appendChild(this.$createElement('span', { className: 'tree-text', innerHTML: text }));
        return node;
      },
      getIndeterminate: function (children, boo) {
        return children.some(function (child) {
          if (child[this.checkedProperty] === boo) {
            return true;
          } else {
            if (child[this.childrenSign] && child[this.childrenSign].length) {
              return this.getIndeterminate(child[this.childrenSign], boo);
            } else {
              return false;
            }
          }
        }, this)
      },
      getTreeNodeChild: function (content) {
        var node = this.$createElement('div', { className: 'tree-node-child' });
        node.appendChild(content);
        return node;
      },
      clickHandler: function (event) {
        var target = event.target;
        if (target.tagName === 'SPAN') {
          var clickChildrenDom = target.parentNode.nextSibling;
          if (!clickChildrenDom) return;
          if (clickChildrenDom.style.display === 'block') {
            clickChildrenDom.style.display = 'none';
            target.classList.remove('expened');
          } else {
            clickChildrenDom.style.display = 'block';
            target.classList.add('expened');
          }
        } else if (target.tagName === 'INPUT') {
          this.checkboxClickHandler(target);
        }
      },
      $createElement: function (tagName, attrs) {
        var node = document.createElement(tagName);
        for (var attr in attrs) {
          if (attrs.hasOwnProperty(attr)) {
            node[attr]= attrs[attr];
          }
        }
        return node;
      },
      _getSelectedNodes: function (leafNodeOnly, exceptIndeterminate) {
        var input = this.node.querySelectorAll('input');
        var selectedData = [];
        input.forEach(function (value) {
          if (leafNodeOnly) {
            if (value.checked && !value.parentNode.nextSibling) {
              selectedData.push(this.dataMap[value.name]);
            }
          } else {
            if (exceptIndeterminate) {
              if (value.checked && !value.indeterminate) {
                selectedData.push(this.dataMap[value.name]);
              }
            } else {
              if (value.checked) {
                selectedData.push(this.dataMap[value.name]);
              }
            }
          }
        }, this)
        return selectedData;
      },
      checkboxClickHandler: function (target) {
        var clickChildrenDom = target.parentNode.nextSibling;
        var parentDom = target.parentNode.parentNode.parentNode;
        var indeterminate = false;
        if (target.checked === true) {
        // 自身被选中
          this.dataMap[target.name][this.checkedProperty] = true;
          if (clickChildrenDom) {
            // 子元素全部选中 
            clickChildrenDom.querySelectorAll('input').forEach(function (value) {
              value.checked = true;
              this.dataMap[value.name][this.checkedProperty] = true;
            }, this)
          }
          // 处理父元素逻辑 
          this.handleParentChange(parentDom, true, false);
        } else {
        // 自身被移除
          this.dataMap[target.name][this.checkedProperty] = false;
          if (clickChildrenDom) {
            // 子元素全部移除
            clickChildrenDom.querySelectorAll('input').forEach(function (value) {
              value.indeterminate = false;
              value.checked = false;
              this.dataMap[value.name][this.checkedProperty] = false;
            }, this)
          }
          // 处理父元素逻辑
          this.handleParentChange(parentDom, false, false);
        }
      },
      handleParentChange: function (parentDom, checked, childIndeterminate) {
        if (!parentDom.previousSibling) return;
        var parentInput = parentDom.previousSibling.querySelector('input');
        // 默认无半选状态
        var indeterminate = false;
        if (checked) {
          // 元素自身被选中， 检查兄弟节点是否全选
          for (var i = 0; i < parentDom.children.length; i++) {
            var inputArray = Array.prototype.slice.call(parentDom.children[i].querySelectorAll('input'))
            var indStatus = inputArray.some(function (value) {
              return !value.checked
            })
            if (indStatus) {
              indeterminate = true;
              break;
            }
          }
          // 赋值状态
          parentInput.checked = true;
          parentInput.indeterminate = childIndeterminate || indeterminate;
          if (!parentInput.indeterminate) {
            this.dataMap[parentInput.name][this.checkedProperty] = true;
          }
        } else {
          // 元素自身被移除， 检查兄弟节点是否全选
          for (var i = 0; i < parentDom.children.length; i++) {
            if (parentDom.children[i].children[0].querySelectorAll('input')[0].checked) {
              // 如果有被选中的，则半选状态并跳出循环
              indeterminate = true;
              break;
            }
          }
          // 赋值状态
          parentInput.checked = indeterminate;
          this.dataMap[parentInput.name][this.checkedProperty] = true;
          parentInput.indeterminate = childIndeterminate || indeterminate;
          this.dataMap[parentInput.name][this.checkedProperty] = false;
        }
        if (parentDom.parentNode.parentNode.className === 'tree-node-child') {
          // 递归父节点
          this.handleParentChange(parentDom.parentNode.parentNode, checked, indeterminate);
        }
      }
    }
    return checkboxTree;
}));
